!
!
!
! Copyright (C) 2002-2008 J. K. Dewhurst, S. Sharma and C. Ambrosch-Draxl.
! This file is distributed under the terms of the GNU General Public License.
! See the file COPYING for license details.
!
!BOP
! !ROUTINE: symvect
! !INTERFACE:
!
!
Subroutine symvect (tspin, vc)
! !USES:
      Use modmain
! !INPUT/OUTPUT PARAMETERS:
!   tspin : .true. if the global spin rotations should be used instead of the
!           spatial rotations (in,logical)
!   vc    : vectors in Cartesian coordinates for all atoms (in,real(3,natmtot))
! !DESCRIPTION:
!   Symmetrises a 3-vector at each atomic site by rotating and averaging over
!   equivalent atoms. Depending on {\tt tspin}, either the spatial or spin part
!   of the crystal symmetries are used.
!
! !REVISION HISTORY:
!   Created June 2004 (JKD)
!   Modified for spin rotations, May 2008 (JKD)
!EOP
!BOC
      Implicit None
! arguments
      Logical, Intent (In) :: tspin
      Real (8), Intent (Inout) :: vc (3, natmtot)
! local variables
      Integer :: is, ia, ja, ias, jas
      Integer :: isym, lsp
      Real (8) :: v (3), t1
! automatic arrays
      Real (8) :: vs (3, natmtot)
! make symmetric average
      vs (:, :) = 0.d0
      Do isym = 1, nsymcrys
         lsp = lsplsymc (isym)
         If (tspin) lsp = lspnsymc (isym)
         Do is = 1, nspecies
            Do ia = 1, natoms (is)
               ias = idxas (ia, is)
               ja = ieqatom (ia, is, isym)
               jas = idxas (ja, is)
               Call r3mv (symlatc(:, :, lsp), vc(:, jas), v)
               vs (:, ias) = vs (:, ias) + v (:)
            End Do
         End Do
      End Do
! normalise
      t1 = 1.d0 / dble (nsymcrys)
      vc (:, :) = t1 * vs (:, :)
      Return
End Subroutine
!EOC
